c compares the RMS deviation of real data with RMS from synthetic data
c   which is generated as either
c  uses two power law functions, white noise, and band-pass filtered white noise
c
c  USED IN COMPANION with est_noise6
c
c
c  1) program 1st computes the 'wander' for the real data more or less defined
c      by Agnew, (1994?) as function of averaging interval
c      averaging interval
c  2)  then loops through as series of synthetic data having the
c       same sampling scheme as the real data
c       computes both wander  of the synthetic data
c  3) outputs the wander for the real data, the statistic from synthetic data.
c       The statists are the confidence interval (68, 90, and 95) based upon ranking the
c        results of simulation. In addition, the chi^2 confidence is estimated

c  Modified April 2004 for differences between est_noise5fd and est_noise5 in computing
c    covariances.
c
      dimension d(45001), t(45001), tim_wan(150),wan_dat(150),H(131072),
     & ran1(131072),ran(131072),dat(131072),wan_sim(150)
      dimension save_wan(301,150),temp(301)
      double precision pl1(131072),pl2(131072)
      character*60 filename,name
      max_wan=150
      max_flt=131072
      max_data=45001
      max_loop=301
c missing data
      fmiss=99999.
      print*,' input the file name of data residuals'
      print*,'  format of data yr jul_day data; 1999. 80.345 8.1'
      read(5,101) filename
101   format(a60)
      open(21,file=filename)
      ic=0
1     continue
        read(21,*,end=2)fyr,fjul,dist
        ic=ic+1
        call jul2day(int(fyr),int(fjul),itime)
        t(ic)=float(itime)+fjul-int(fjul)
        d(ic)=dist
      go to 1
2     continue
      close (21)
      print*,' Number of observations read',ic
      if (ic .gt. max_data) then
        print*,' Number of data exceeds array dimension of',max_data
        stop
      end if
      print*,' input name of file with noise parameters'
      read(5,101)filename
      open(22,file=filename)
      read(22,7879)name,fmax,idec,iccc,nmod,
     &  siginstr,
     &  amp1,    
     &  exp1,     
     &  alpha,    
     &  amp_bp,    
     &  amp2,    
     &  exp2
7879  format(a20,3x,f10.3,3i6,7(3x,f10.3,1x,10x))
      close(22) 
c  get smallest sampling interval
      dt_sam=999999.
      do 3 i=2,ic
3     if (t(i)-t(i-1) .lt. dt_sam) dt_sam=t(i)-t(i-1)
      print*,' Sampling interval is ',dt_sam,' days'
      print*,' sigma=',siginstr
      print*,' exp1=',exp1
      print*,' plamp1=',amp1
      print*,' GM=',alpha
      print*,' exp2=',exp2
      print*,' plamp2=',amp2
c
c  calculate the so called "wander" of the data for
c  specified periods
c
c  determine the periods; use constant log(time) to
c   increment the periods
c
      tmin=dt_sam
c      tmin=0.25
      tmax=(t(ic)-t(1))
      dtt=0.05
4     continue
      dtt1=1/dtt
c      nt=int(dtt1*(log10(tmax)-log10(tmin) )) + 1
      nt=int(dtt1*(log10(tmax)-log10(2.0*tmin) )) + 1
      print*, 'nt =',nt
      do 5 i=1,nt
c        tlog=float(i-1)*dtt+log10(tmin)
        tlog=float(i-1)*dtt+log10(2.0*tmin)
        if (i .eq. 1) then
          tim_wan(i)=10**tlog
c          print*,i,tlog,tim_wan(i)
          tlast=tim_wan(i)
          ntt=1
        else
          t_test=10**tlog
          if (t_test-tlast .ge. tmin) then
            ntt=ntt+1
            nx=int(t_test/tmin)
            t_test=float(nx)*tmin
            tim_wan(ntt)=t_test
            tlast=t_test
c            print*,ntt,i,tlog,tim_wan(ntt)
          end if
        end if
5     continue
      print*,' Number of intervals to compute wander is:', ntt
      if (ntt .gt. max_wan) then
c  test if number of wander periods exceeds dimension; if so, increase spacing
        dtt=dtt*1.5
        go to 4
      end if
c
c  change data spacing 
c
      do 6 i=1,max_flt
6     dat(i)=fmiss
      do 7 i=1,ic
      ix=int((t(i)-t(1))/dt_sam) + 1
      dat(ix)=d(i)
7     continue
      ix_max=int((t(ic)-t(1))/dt_sam) + 1
      if (ix_max .gt. max_flt) then
        print*, 'ix_max=', ix_max,' exceeds max_flt=', max_flt
        stop
      end if

      call wander(tim_wan,wan_dat,max_wan,ntt,dat,max_flt,ix_max,
     &  dt_sam)
c
c  revise list of tim_wan for missing data
c
      nx=ntt
      do 10 i=1,ntt
      if (wan_dat(i) .eq. fmiss) then
        nx=nx-1
        do 11 j=i,nx
          tim_wan(j)=tim_wan(j+1)
          wan_dat(j)=wan_dat(j+1)
11      continue
      end if
10    continue
      if (nx .ne. ntt) then
        print*,' found ',ntt-nx,' periods without estimating wander'
        ntt=nx
      end if
      print*,' Wander**2 of data is'
      do 8 i=1,ntt
8     print*,tim_wan(i),wan_dat(i)
c
c Simulate data using noise model
c
c  read in number of simulations
c
      print*,' Input the number of sets of sythetic data for'
     &  ,' simulations'
      read(5,*)nloop
      if (nloop .gt. max_loop) then
         print*,' Number of specified loops exceeds dimensions',
     &    max_loop
         nloop=max_loop
      end if
c
c  determine filter coeficients for 2 power law noise models, and 1 band-pass filter
c     white noise
c
      print*,' For bandpass filtered noise, input the pass bands;'
      print*,'   flow and fhigh is c/yrs'
      read(5,*)fl,fh
      if (fl .gt. fh) then
        ftemp=fl
        fl=fh
        fh=ftemp
      end if
      print*,'  and input the number of poles'
      read(5,*)np
      ts=dt_sam/365.25
      npts=ix_max
c      amp_bp=1.
      print*,' IN BANDPASS', amp_bp,ts,npts+100,fl,fh,np
      call band_pass_filt(ts,fl,fh,np,max_flt,npts+100,H,amp_bp)

      call frac_diff(pl1,exp1,alpha,ts,npts+100,max_flt)
      call frac_diff(pl2,exp2,0.0,ts,npts+100,max_flt)
      open(23,file='seed.dat')
      read(23,*)iseed

c      nloop=1
      do 30 il=1,nloop
c convolve white noise with each of the filters to create simulated data
c
c zero-out data
        do 31 i=1,ix_max
31      if (dat(i) .ne. fmiss) dat(i)=0.

        if (siginstr .ne. 0) then
c  create white noise
c          print*,'white'
          call gaus(ran1,max_flt,iseed)
          do 32 i=1,ix_max
32        if (dat(i) .ne. fmiss) dat(i)=dat(i)+siginstr*ran1(i)
        end if
         
c convolve with first power law
        if (amp1 .ne. 0) then
c          print*,'PL1'
          call gaus(ran1,max_flt,iseed)
          call convol_pl(ran1,max_flt,pl1,ran,ts,amp1,exp1,npts)
          do 33 i=1,ix_max
33        if (dat(i) .ne. fmiss) dat(i)=dat(i)+ran(i)

c          do 88 i=1,npts
c88        write(7,*)i,ran(i)
        end if
c convolve with second power law
        if (amp2 .ne. 0) then
          call gaus(ran1,max_flt,iseed)
          call convol_pl(ran1,max_flt,pl2,ran,ts,amp2,exp2,npts)
          do 34 i=1,ix_max
34        if (dat(i) .ne. fmiss) dat(i)=dat(i)+ran(i)

c          do 89 i=1,npts
c89        write(8,*)i,ran(i)
        end if
c convolve bandpass filtered noise
        if (amp_bp .ne. 0) then
          call gaus(ran1,max_flt,iseed)
          call convol_bp(ran1,max_flt,H,ran,ts,npts)
          do 35 i=1,ix_max
35        if (dat(i) .ne. fmiss) dat(i)=dat(i)+ran(i)

c          do 890 i=1,npts
c890        write(9,*)i,ran(i)
        end if
c  variance of data
        var=0
        do 38 i=1,npts
38      if (dat(i) .ne. fmiss) var=var+dat(i)**2
        sd=sqrt(var/float(ic))
        print*,'For synthetic data ',il,' Standard deviation is',sd
c  Calculate wander of synthetic data

        call wander(tim_wan,wan_sim,max_wan,ntt,dat,max_flt,ix_max,
     &  dt_sam)
c      print*,' Wander**2 of synthetic data is'
c      do 887 i=1,ntt
c887     print*,tim_wan(i),wan_dat(i)
c      do 891 i=1,ix_max
c891   if (dat(i) .ne. fmiss) write(10,*),i,dat(i)

c  save the results
       do 36 i=1,ntt
36     save_wan(il,i)=wan_sim(i)


30    continue

      rewind(23)
      iseed=int(ran1(max_flt)*100000)
      write(23,*)iseed
      close(23)
c
c Done with computing wander of synthetic data
c
c  for each period, put the wander in sequential order to do statistics
c
      do 40 j=1,ntt
        do 41 i=1,nloop
41      temp(i)=save_wan(i,j)
        call chron(temp,nloop,max_loop)
        do 42 i=1,nloop
42      save_wan(i,j)=temp(i)
40    continue
      n95=int(nloop*0.025 +0.5)
      n90=int(nloop*0.05 + 0.5)
      n68=int(nloop*0.16 + 0.5)
      n50=int(nloop*0.50 + 0.5)
      print*,' '
      print*,'for wander'
      print*,'68% confidence intervals between ',n68,' and ',nloop-n68
      print*,'90% confidence intervals between ',n90,' and ',nloop-n90
      print*,'95% confidence intervals between ',n95,' and ',nloop-n95
      print*,' median is ',n50
      open(88,file='wander.out')
c
c  do statistics on the simulated wander and compare with wander from data
c
      print*,' Program will compute a chi^2 statistic of fit to'
      print*,'  compare simulated wander spectra with actual wander.'
      print*,'  To facilate closer examination of the fit, the'
      print*,'  comparison is broken down into short period and'
      print*,'   long period chi^2.'
      print*,' Input the dividing period (days) between the long ',
     &  'and short term wander'
      read(5,*)tau
      RMS_wan=0.
      RMS_wan_long=0.
      RMS_wan_short=0.
      n_wan_short=0
      n_wan_long=0
      n_wan=0.
      do 50 j=1,ntt
        xsum=0
        x2sum=0
        xsuml=0
        x2suml=0
        nCI=0
        iflag_CI=0
        do 51 k=1,nloop

          if (save_wan(k,j) .gt. wan_dat(j)) then
            if (iflag_CI .eq. 0) then
            iflag_CI=1
            nCI=k
            end if
          end if
          if (nCI .eq. 1) conf_i=-float(nloop-2)/float(nloop)
          if (nCI .eq. 0) conf_i=float(nloop-2)/float(nloop)
          if (nCI .gt. 1) then
c           interpolate:
             conf_i=(2/float(nloop) -1) + float(nCI)*(2/float(nloop))
     &    -(2/float(nloop))
     &    *(wan_dat(j)-save_wan(nCI,j))
     &     /(save_wan(nCI-1,j)-save_wan(nCI,j))
          end if
          xsum=xsum+(save_wan(k,j))
          x2sum=x2sum+save_wan(k,j)**2
          xsuml=xsuml+log10(save_wan(k,j))
          x2suml=x2suml+(log10(save_wan(k,j)))**2
51      continue
        wan_mean=xsum/float(nloop)
        wan_var=sqrt((x2sum-float(nloop)*wan_mean*wan_mean)
     &     /float(nloop-1))
        wan_meanl=xsuml/float(nloop)
        wan_varl=sqrt((x2suml-float(nloop)*wan_meanl*wan_meanl)
     &     /float(nloop-1))
        write(88,883)tim_wan(j),sqrt(wan_dat(j)),
     &   sqrt(save_wan(n68,j)),sqrt(save_wan(nloop-n68,j)),
     &   sqrt(save_wan(n90,j)),sqrt(save_wan(nloop-n90,j)),
     &   sqrt(save_wan(n95,j)),sqrt(save_wan(nloop-n95,j)),
     &   sqrt(wan_mean),sqrt(wan_var),conf_i*100
883     format(f13.4,9f12.4,f8.1)
c  accummulate chi^2 statitistic
        n_wan=n_wan+1
c   compare wan_varl and the log of the 68% CI
        flog_hi=log10(abs(save_wan(n68,j)))
     &     -log10(abs(save_wan(n50,j)))
        flog_low=log10(abs(save_wan(nloop-n68,j)))
     &    -log10(abs(save_wan(n50,j)))
        ave_log=(abs(flog_hi)+abs(flog_low))/2.0
        
        sgg=1
        if (ave_log .gt. wan_varl) sgg=ave_log/wan_varl
        print*,tim_wan(j),wan_varl,ave_log,flog_low,flog_hi,sgg
        RMS_wan=RMS_wan+
     &  (((log10(wan_dat(j)))-wan_meanl)/(sgg*wan_varl))**2
c        print*,tim_wan(j),log10(wan_dat(j)),wan_meanl,n_wan,RMS_wan
        if (tim_wan(j) .le. tau) then
          RMS_wan_short=RMS_wan_short
     &      +(((log10(wan_dat(j)))-wan_meanl)/wan_varl)**2
          n_wan_short=n_wan_short+1
        else
          RMS_wan_long=RMS_wan_long
     &      +(((log10(wan_dat(j)))-wan_meanl)/wan_varl)**2
          n_wan_long=n_wan_long+1
        end if
50    continue
      print*,' '
      print*,' data wander and confidence limits for plotting ',
     &  'in "wander.out" '
      print*,' col 1  is time interval in days, col 2 is ',
     &  'sqrt of data wander'
      print*,' col 3 and 4 is 68% CI, 5 and 6 is 90%, ',
     &   ' and 7 and 8 is 95%'
      print*,'  col 9 is the average, col 10 is the standard deviation'
      print*,'  col 11 is the confidence that the real data deviates',
     &  ' from simulated data'
      close(88)
      print*,' '
      print*,' Reduced Chi square for ',n_wan, ' obs. of wander: '
     &   ,sqrt(RMS_wan/float(n_wan))
      Q_wan=gammq(0.5*n_wan,0.5*RMS_wan)
      P_wan=gammp(0.5*n_wan,0.5*RMS_wan)
      print*,' For all, chi^2 probs for wander is ',Q_wan,P_wan
      print*,'     2nd number is rejection confidence'
      print*,' '
      print*,' Reduced Chi square for periods less equal to', tau,
     &  ' days',
     & sqrt(RMS_wan_short/float(n_wan_short))
      print*,' num of pts: ',n_wan_short
      Q_wan=gammq(0.5*n_wan_short,0.5*RMS_wan_short)
      P_wan=gammp(0.5*n_wan_short,0.5*RMS_wan_short)
      print*,' For short, chi^2 probs for wander is ',Q_wan,P_wan
      print*,'     2nd number is rejection confidence'
      print*,' '
      print*,' Reduced Chi square for periods more than', tau,' days',
     & sqrt(RMS_wan_long/float(n_wan_long))
      print*,' num of pts: ',n_wan_long
      Q_wan=gammq(0.5*n_wan_long,0.5*RMS_wan_long)
      P_wan=gammp(0.5*n_wan_long,0.5*RMS_wan_long)
      print*,'For long, chi^2 probs for wander is ',Q_wan,P_wan
      print*,'     2nd number is rejection confidence'
      print*,' '
      print*,' '


      stop
      end 
      subroutine chron(t,ic,max)
c  puts data into chronological order
c  code riped-off from numerical recipes piksrt.f
      dimension t(max)
      do 12 j=2,ic
         tx=t(j)

         do 11 i=j-1,1,-1
         if (t(i) .eq. tx) t(i)=t(i)+0.001
         if (t(i) .lt. tx) go to 10
           t(i+1)=t(i)
11       continue
         i=0
10       t(i+1)=tx
12    continue
      return
      end

      subroutine convol_bp(ran1,max_flt,crr,ran,ts,npts)
c  does convolution for bandpass stuff
      dimension ran1(max_flt),ran(max_flt),crr(max_flt)
         do 44 i=1,npts+100
         sum=0.
           do 440 j=1,i
           sum=sum+crr(j)*ran1(1+i-j)
440        continue
         ran(i)=sum
44       continue
c  shift data over by 100 points to remove 1st set of numbers
        do 1 i=1,npts
1       ran(i)=ran(i+100)
      return
      end 
      subroutine convol_pl(ran1,max_flt,crr,ran,ts,amp,expon,npts)
c  does convolution for power law stuff
      dimension ran1(max_flt),ran(max_flt)
      double precision crr(max_flt)
         do 44 i=1,npts+100
         sum=0.
           do 440 j=1,i
           sum=sum+crr(j)*ran1(1+i-j)
440        continue
         ran(i)=amp*sum* ((ts)**(expon/4) )
44       continue
c  shift data over by 100 points to remove 1st set of numbers
        do 1 i=1,npts
1       ran(i)=ran(i+100)
      return
      end 

      subroutine frac_diff(crr,expon,alpha,ts,npts,nmax)
c use Hosking routine to create fractional differences
c  crr are the fractional difference
c  expon is the exponent
c  alpha is the GM freq in radians/yr
c  ts is the sampling interval in years
c  npts is length of time series
c  nmax is dimension of crr
      double precision crr(nmax), small
      ddexp=0.5*expon
      small=1.0d-10
         do 39 j=1,npts+100
         if (j .eq. 1) fracti=1.0
         if (j .eq. 2) then
            fracti=ddexp
            fracti_last=fracti
         end if
         if (j .ge. 3) then
            fracti=fracti_last*(ddexp+float(j-2))/float(j-1)
            fracti_last=fracti
         end if
c         crr(j)=fracti*((1.0-alpha*ts)**((j)))
         crr(j)=fracti*dexp(-dble(alpha)*ts*float(j-1))
         if (crr(j) .lt. small) crr(j)=small     
39       continue
      return
      end
      subroutine band_pass_filt(ts,fl,fh,np,nmax,npts,H,amp)
c create a bandpass filter in the time domain
c  define filter in frequency domain then take inverse fft of filter
c    tranfer function to get time domain impulse responce
c  ts is sampling interval in years
c  fl and fh are low and high freq break pts in c/yr
c  np is number of poles 1 to 4
c  nmax is the dimension
c  npts is the number of points in time series
c  H on output is the impulse responce
c  amp is the amplitude of filter.
      dimension H(nmax)
      complex hc(nmax),h1,h2,h3,h4
      fh=fh*ts
      fl=fl*ts
      fn=0.5
      H(1)=1.
      H(1)=0.
      len=npts+100
      nf=int(len/2)
      nf=int((log(float(len/2))/log(2.))+0)
      nf=nf+1
      nf=2**nf
c      print*,' Number of frequencies ',nf
      len=2*nf
c      print*,' length of time series for band-pass noise is ',len
c      print*,ts,fl,fh,np,amp
      peak=-99999.
      do 50 i=1,nf
        f=(1/float(len))*(i)
        h1=cmplx(1.0,0.0)
        h2=cmplx(0.,f/fl)
        h3=cmplx(1.0,f/fl)
        h4=cmplx(1.,f/fh)
        hc(i)=h1*h2/(h3*h4)
        if (np .eq. 2) hc(i)=hc(i)*hc(i)
        if (np .eq. 3) hc(i)=hc(i)*hc(i)*hc(i)
        if (np .eq. 4) hc(i)=hc(i)*hc(i)*hc(i)*hc(i)
        if (np .eq. 5) hc(i)=hc(i)*hc(i)*hc(i)*hc(i)*hc(i)
        if (np .eq. 6) hc(i)=hc(i)*hc(i)*hc(i)*hc(i)*hc(i)*hc(i)
        H(i*2+1)=real(hc(i))
        H(i*2+2)=-aimag(hc(i))
        if (cabs(hc(i)) .gt. peak) then
           peak=cabs(hc(i))
           f_peak=f/ts
        end if
c      write(70,100)f/ts,real(hc(i)),-aimag(hc(i)),
c     & 20*alog10(cabs(hc(i))),
c     &   57.2958*atan2(real(hc(i)),-aimag(hc(i))),i*2+1,i*2+2
100   format(5f15.5,2i6)
50    continue
c      peak=peak
      H(2)=cabs(hc(nf))

c      write(70,100)float(1/(ts*2)),H(2),0,20*alog10(H(2))
c      print*,' peak gain is ',20*log10(peak),' db  at ',f_peak,' c/yr'
      fo=0.5*(fl+fh)
      calc_amp=(fo/fl)/(sqrt(1.0 + (fo/fl)**2)*sqrt(1.0 + (fo/fh)**2))
      calc_amp=(calc_amp**np)
c      print*,' Calculated peak gain ',20*log10(calc_amp), 
c     &  ' at ',fo/ts,' c/yr'
c      amp=amp/peak
cc      print*,' Normalized to ',20*log10(amp)+20*log10(peak), ' db'
cc      print*,' Factor in amplitude; gain is: ',
cc     &  20*log10(amp*peak), ' db'
cc      print*,' Final peak amplitude ',20*log10(amp) ,' db'
cc
cc  compute inverse transform
cc
      call realft(H,len,-1)
      do 51 i=1,len
c        write(80,*)amp*H(i)*2.0/float(len)
        H(i)=((amp/peak)*H(i)*2.0/float(len))*sqrt(0.5/ts)
51     continue
c       print*,' Amplitude of BP-noise: ',20*log10(amp/calc_amp),
c     &    ' db at ',365.25*fo,' c/yr'
      fh=fh/ts
      fl=fl/ts
c      amp=amp*peak

       return
       end
      subroutine wander(tim_wan,wan_data,max_wan,ntt,dat,max_flt,ix,
     &  dt_sam)

c  calculates the wander for specificed intervals
c   wan_data(T)=sum (dat(T+t)-dat(t))^2 / N
c  tim_wan are the time interval (days) to estimate wander--input
c  wan_dat are the estimates of wander for specified tim_wan--output
c  dat observations (missing data 99999)
c  max_wan and max_flt  array dimensions
c  ntt  number of periods in tim_wan
c  ic number of data
c  dt_sam sampling interval in days

      dimension tim_wan(max_wan),wan_data(max_wan),
     &  dat(max_flt)
c  missing data is 99999.
      fmiss=99999.
      do 10 j=1,ntt
        wan_data(j)=0.0
        n=0
        tau=tim_wan(j)
        itau=int(tau/dt_sam)
        if (j .eq. 1) itau_min=int(tim_wan(1)/dt_sam)
        if (j .gt. 1) itau_min=int((tim_wan(j-1))/dt_sam +1)
        itau_max=int(tim_wan(j)/dt_sam)
c        print*,tim_wan(j),itau_min,itau_max
        do 151 k_tau=itau_min,itau_max
          do 152 i=1,ix-k_tau
            if ((dat(i) .ne. fmiss ) .and. 
     &        (dat(i+k_tau) .ne. fmiss)) then
              n=n+1
             wan_data(j)=wan_data(j)+(dat(i+k_tau)-dat(i))**2
           end if
152       continue
151     continue
        if (n .ne. 0) wan_data(j)=(wan_data(j)/float(n))
        if (n .eq. 0) wan_data(j)=fmiss
c      print*,j,tim_wan(j),itau,itau_min,itau_max,n,wan_data(j)

10    continue
      return
      end

      subroutine gausX(x,nd,ir)
c  Old version; not used
c  program creates white noise with a gausian distribution with
c  variance=1 and mean equal to zero
c  x are an array of random numbers
c  nd is the length of the array
c  ir is the seed for the random number generator
      dimension x(1)
c      x(1)=rand(ir)
      ir=0
      do 15 j=1,nd
      xd=0.
        do 13 i=1,12
c13      xd=xd+rand(ir)
13      xd=xd+1
15    x(j)=xd-6.0
      sum=0.
      do 20 i=1,nd
      sum=x(i)+sum
20    continue
      xmean=sum/float(nd)
      sum2=0.
      do 30 i=1,nd
      x(i)=x(i)-xmean
      sum2=sum2+x(i)**2
30    continue
      sd=sqrt(sum2/float(nd-1))
      do 40 i=1,nd
      x(i)=x(i)/sd
40    continue
      return
      end
      FUNCTION gammp(a,x)
      REAL a,gammp,x
CU    USES gcf,gser
      REAL gammcf,gamser,gln
      if(x.lt.0..or.a.le.0.)pause 'bad arguments in gammp'
      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software YaAV.
      FUNCTION gammq(a,x)
      REAL a,gammq,x
CU    USES gcf,gser
      REAL gammcf,gamser,gln
      if(x.lt.0..or.a.le.0.)pause 'bad arguments in gammq'
      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammq=1.-gamser
      else
        call gcf(gammcf,a,x,gln)
        gammq=gammcf
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software YaAV.
      SUBROUTINE gcf(gammcf,a,x,gln)
      INTEGER ITMAX
      REAL a,gammcf,gln,x,EPS,FPMIN
      PARAMETER (ITMAX=200,EPS=3.e-7,FPMIN=1.e-30)
c      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)
CU    USES gammln
      INTEGER i
      REAL an,b,c,d,del,h,gammln
      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d
      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b
        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c
        if(abs(c).lt.FPMIN)c=FPMIN
        d=1./d
        del=d*c
        h=h*del
        if(abs(del-1.).lt.EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gcf'
1     gammcf=exp(-x+a*log(x)-gln)*h
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software YaAV.
      SUBROUTINE gser(gamser,a,x,gln)
      INTEGER ITMAX
      REAL a,gamser,gln,x,EPS
      PARAMETER (ITMAX=100,EPS=3.e-7)
CU    USES gammln
      INTEGER n
      REAL ap,del,sum,gammln
      gln=gammln(a)
      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in gser'
        gamser=0.
        return
      endif
      ap=a
      sum=1./a
      del=sum
      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS)goto 1
11    continue
      pause 'a too large, ITMAX too small in gser'
1     gamser=sum*exp(-x+a*log(x)-gln)
      return
      END
      function gammln(xx)
      real gammln,xx
      integer j
      double precision ser,stp,tmp,x,y,cof(6)
      save cof,stp
      data cof,stp/76.18009172947146d0,-86.50532032941677d0,
     & 24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     & -.5395239384953d-5,2.5066282746310005d0/
      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue
      gammln=tmp+log(stp*ser/x)
      return
      end
      SUBROUTINE realft(data,n,isign)
      INTEGER isign,n
      REAL data(n)
CU    USES four1
      INTEGER i,i1,i2,i3,i4,n2p3
      REAL c1,c2,h1i,h1r,h2i,h2r,wis,wrs
      DOUBLE PRECISION theta,wi,wpi,wpr,wr,wtemp
      theta=3.141592653589793d0/dble(n/2)
      c1=0.5
      if (isign.eq.1) then
        c2=-0.5
        call four1(data,n/2,+1)
      else
        c2=0.5
        theta=-theta
      endif
      wpr=-2.0d0*sin(0.5d0*theta)**2
      wpi=sin(theta)
      wr=1.0d0+wpr
      wi=wpi
      n2p3=n+3
      do 11 i=2,n/4
        i1=2*i-1
        i2=i1+1
        i3=n2p3-i2
        i4=i3+1
        wrs=sngl(wr)
        wis=sngl(wi)
        h1r=c1*(data(i1)+data(i3))
        h1i=c1*(data(i2)-data(i4))
        h2r=-c2*(data(i2)+data(i4))
        h2i=c2*(data(i1)-data(i3))
        data(i1)=h1r+wrs*h2r-wis*h2i
        data(i2)=h1i+wrs*h2i+wis*h2r
        data(i3)=h1r-wrs*h2r+wis*h2i
        data(i4)=-h1i+wrs*h2i+wis*h2r
        wtemp=wr
        wr=wr*wpr-wi*wpi+wr
        wi=wi*wpr+wtemp*wpi+wi
11    continue
      if (isign.eq.1) then
        h1r=data(1)
        data(1)=h1r+data(2)
        data(2)=h1r-data(2)
      else
        h1r=data(1)
        data(1)=c1*(h1r+data(2))
        data(2)=c1*(h1r-data(2))
        call four1(data,n/2,-1)
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software YaAV.
      SUBROUTINE four1(data,nn,isign)
      INTEGER isign,nn
      REAL data(2*nn)
      INTEGER i,istep,j,m,mmax,n
      REAL tempi,tempr
      DOUBLE PRECISION theta,wi,wpi,wpr,wr,wtemp
      n=2*nn
      j=1
      do 11 i=1,n,2
        if(j.gt.i)then
          tempr=data(j)
          tempi=data(j+1)
          data(j)=data(i)
          data(j+1)=data(i+1)
          data(i)=tempr
          data(i+1)=tempi
        endif
        m=n/2
1       if ((m.ge.2).and.(j.gt.m)) then
          j=j-m
          m=m/2
        goto 1
        endif
        j=j+m
11    continue
      mmax=2
2     if (n.gt.mmax) then
        istep=2*mmax
        theta=6.28318530717959d0/(isign*mmax)
        wpr=-2.d0*sin(0.5d0*theta)**2
        wpi=sin(theta)
        wr=1.d0
        wi=0.d0
        do 13 m=1,mmax,2
          do 12 i=m,n,istep
            j=i+mmax
            tempr=sngl(wr)*data(j)-sngl(wi)*data(j+1)
            tempi=sngl(wr)*data(j+1)+sngl(wi)*data(j)
            data(j)=data(i)-tempr
            data(j+1)=data(i+1)-tempi
            data(i)=data(i)+tempr
            data(i+1)=data(i+1)+tempi
12        continue
          wtemp=wr
          wr=wr*wpr-wi*wpi+wr
          wi=wi*wpr+wtemp*wpi+wi
13      continue
        mmax=istep
      goto 2
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software YaAV.
      subroutine gaus(x,nd,ir)
c  program creates white noise with a gausian distribution with
c  variance=1 and mean equal to zero
c  x are an array of random numbers
c  nd is the length of the array
c  ir is the seed for the random number generator
      dimension x(1)
      if (ir .gt. 0) ir=-1*ir
      do 1 i=1,nd
      x(i)=gasdev(ir)
1     continue
      return
      end
      FUNCTION GASDEV(IDUM)
c From Numerical Recipes; 1992
      SAVE ISET,GSET
      DATA ISET/0/
      IF (ISET.EQ.0) THEN
1       V1=2.*RAN1(IDUM)-1.
        V2=2.*RAN1(IDUM)-1.
        R=V1**2+V2**2
        IF(R.GE.1. .or. r .eq. 0)GO TO 1
        FAC=SQRT(-2.*LOG(R)/R)
        GSET=V1*FAC
        GASDEV=V2*FAC
        ISET=1
      ELSE
        GASDEV=GSET
        ISET=0
      ENDIF
      RETURN
      END
      FUNCTION RAN1(IDUM)
c From Numerical Recipes; 1992
c idum must be less than zero to initialize
      integer idum,IA,IM,IQ,IR,NTAB,NDIV
      REAL ran1,AM,EPS,RNMX
      PARAMETER (IA=16807,IM=2147483647,AM=1./IM,IQ=127773,IR=2836,
     &   NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=1.2e-07,RNMX=1.-EPS)
      INTEGER j,k,iv(NTAB),iy
      SAVE iv,iy
      DATA iv /NTAB*0/, iy /0/
      if (idum .le. 0 .or. iy .eq. 0) then
        idum=max(-idum,1)
        do 11 j=NTAB+8,1,-1
          k=idum/IQ
          idum=IA*(idum-k*IQ)-IR*k
          if (idum .lt. 0) idum=idum+IM
          if (j .le. NTAB) iv(j)=idum
11      continue
        iy=iv(1)
      endif
      k=idum/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum .lt. 0) idum=idum+IM
      j=1+iy/NDIV
      iy=iv(j)
      iv(j)=idum
      ran1=min(AM*iy,RNMX)
      return
      end
